from ngsolve import *
from netgen.geom2d import SplineGeometry
from numpy import pi
# from geom import *

def twoDomain():
  geometry = SplineGeometry()
  
  # point coordinates ...
  pnts = [ (0,0), (1,0), (1,1), (0,1), (2,0), (2,1)]
  pnums = [geometry.AppendPoint(*p) for p in pnts]
  # start-point, end-point, boundary-condition, domain on left side, domain on right side:
  lines = [ (pnums[0],pnums[1],1,1,0), (pnums[1],pnums[2],2,1,2), 
       (pnums[2],pnums[3],3,1,0), (pnums[3],pnums[0],4,1,0), (pnums[1], pnums[4],5,2,0),
       (pnums[4],pnums[5],6,2,0), (pnums[5],pnums[2],7,2,0)]
  for p1,p2,bc,left,right in lines:
      geometry.Append( ["line", p1, p2], bc=bc, leftdomain=left, rightdomain=right)
  return geometry

def GetInterfaceIndicator(mesh):
  dom_indicator = GridFunction(L2(mesh,order=0))
  dom_indicator.Set(CoefficientFunction([1,2]))
  
  ffes = FacetFESpace(mesh,order=0)
  dom_ind_average = GridFunction(ffes)
  mf = BilinearForm(ffes,symmetric=True)
  mf += SymbolicBFI(ffes.TrialFunction()*ffes.TestFunction(),element_boundary=True)
  mf.Assemble()
  ff = LinearForm(ffes)
  ff += SymbolicLFI(dom_indicator*ffes.TestFunction(),element_boundary=True)
  ff.Assemble()
  dom_ind_average.vec.data = mf.mat.Inverse() * ff.vec

  interface_indicator = BitArray(ffes.ndof)
  for i in range(len(dom_ind_average.vec)):
    if (dom_ind_average.vec[i] > 1.75) or (dom_ind_average.vec[i] < 1.25):
      interface_indicator[i] = False
    else:
      interface_indicator[i] = True
  print("facets marked as interface facets:", interface_indicator)
  return interface_indicator

# interface number = 2
mesh = Mesh(twoDomain().GenerateMesh (maxh=0.2))


order = 2
uex = sin(pi*x)*sin(pi*y)
source = 2*pi*pi*uex
V1 = H1(mesh, order=order, dirichlet=[1,3,4], definedon=[1])
V2 = H1(mesh, order=order, dirichlet=[5,6,7], definedon=[2])
V = FESpace([V1,V2], flags={"dgjumps":True})

u1,u2 = V.TrialFunction()
v1,v2 = V.TestFunction()

n = specialcf.normal(2)
h = specialcf.mesh_size
interface_indicator = GetInterfaceIndicator(mesh)

dom1 = GridFunction(L2(mesh,order=0))
dom1.Set(CoefficientFunction([1,0]))
dom2 = GridFunction(L2(mesh,order=0))
dom2.Set(CoefficientFunction([0,1]))


a = BilinearForm(V)
a += SymbolicBFI (grad(u1)*grad(v1),definedon=[1])
a += SymbolicBFI (grad(u2)*grad(v2),definedon=[2])

# nitsche term
cf1 = 0.5*(grad(u1)+grad(u2.Other()))*n*(v2.Other()-v1)
cf2 = 0.5*(grad(v1)+grad(v2.Other()))*n*(u2.Other()-u1)
cf3 = 10*order*order/h*(v1 - v2.Other())*(u1 - u2.Other())

interfacebfi = SymbolicBFI (cf1+cf2+cf3, VOL, skeleton=True )
interfacebfi.SetDefinedOnElements(interface_indicator)
a += interfacebfi
a.Assemble()

f = LinearForm(V)
f += SymbolicLFI ( source * v1, definedon = [1])
f += SymbolicLFI ( source * v2, definedon = [2])
f.Assemble()

u = GridFunction(V)
u.vec.data = a.mat.Inverse(V.FreeDofs()) * f.vec
#u.components[0].Set(uex,BND)
#u.components[1].Set(uex,BND)
#res = f.vec.CreateVector()
#res.data = f.vec - a.mat*u.vec
#u.vec.data += a.mat.Inverse(V.FreeDofs()) * res

# project to L2 space
Vpost = L2(mesh, order = order)
up = Vpost.TrialFunction()
vp = Vpost.TestFunction()
ap = BilinearForm(Vpost)
ap += SymbolicBFI(up*vp)
fp = LinearForm(Vpost)
fp += SymbolicLFI(u.components[0]*vp,definedon=[1])
fp += SymbolicLFI(u.components[1]*vp,definedon=[2])

ap.Assemble()
fp.Assemble()
up = GridFunction(Vpost)
up.vec.data = ap.mat.Inverse()*fp.vec

err = sqrt(Integrate((up-uex)*(up-uex),mesh))
print(err)
Draw(up)
